@@ -18,7 +18,7 @@
 
 """This file contains the version of the Translate Toolkit."""
 
-ver = (3, 18, 0)
+ver = (3, 19, 0)
 """Machine readable version number. Used by tools that need to adjust code
 paths based on a Translate Toolkit release number."""
 
@@ -29,5 +29,5 @@ code touching stats or quality checks.  An increased build number will force a
 toolkit user, like Pootle, to regenerate its stored stats and check
 results."""
 
-sver = "{}.{}.{}".format(*ver)
+sver = f"{ver[0]}.{ver[1]}.{ver[2]}"
 """Human readable version number. Used for version number display."""
