@@ -73,10 +73,10 @@ class ical2po:
 
     def merge_stores(self) -> None:
         """Convert two source format files to a target format file."""
-        self.extraction_msg = f"extracted from {self.template_store.filename}, {self.source_store.filename}"
+        self.extraction_msg = f"extracted from {self.template_store.filename}, {self.source_store.filename}"  # ty:ignore[unresolved-attribute]
 
         self.source_store.makeindex()
-        for template_unit in self.template_store.units:
+        for template_unit in self.template_store.units:  # ty:ignore[unresolved-attribute]
             target_unit = self.convert_unit(template_unit)
 
             template_unit_name = "".join(template_unit.getlocations())
