@@ -29,18 +29,18 @@ from translate.storage.base import ParseError
 
 # Override the piclose tag from simple > to ?> otherwise we consume HTML
 # within the processing instructions
-html.parser.piclose = re.compile(r"\?>")
+html.parser.piclose = re.compile(r"\?>")  # ty:ignore[unresolved-attribute]
 
 
 class htmlunit(base.TranslationUnit):
     """A unit of translatable/localisable HTML content."""
 
-    def __init__(self, source=None):
+    def __init__(self, source=None) -> None:
         super().__init__(source)
         self.locations = []
         self._context = ""
 
-    def addlocation(self, location):
+    def addlocation(self, location) -> None:
         self.locations.append(location)
 
     def getlocations(self):
@@ -51,7 +51,7 @@ class htmlunit(base.TranslationUnit):
         """Get the message context."""
         return self._context
 
-    def setcontext(self, context):
+    def setcontext(self, context) -> None:
         """Set the message context."""
         self._context = context or ""
 
@@ -161,7 +161,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         re.VERBOSE | re.IGNORECASE,
     )
 
-    def __init__(self, inputfile=None, callback=None):
+    def __init__(self, inputfile=None, callback=None) -> None:
         super().__init__(convert_charrefs=False)
         base.TranslationStore.__init__(self)
 
@@ -187,6 +187,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         self._ancestor_id_label_stack = []  # labels for ancestor ids (id or id:line-col)
         self._id_seen = set()  # track seen ids to disambiguate labels
         self._units_by_source = {}  # Track units by normalized source to add context only when needed for disambiguation
+        self._units_by_src_ctx = {}  # Fast lookup for units created with explicit context: (source, context) -> unit
 
         # parse
         if inputfile is not None:
@@ -214,7 +215,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         self.guess_encoding(htmlsrc)
         return htmlsrc.decode(self.encoding)
 
-    def parse(self, htmlsrc):
+    def parse(self, htmlsrc) -> None:  # ty:ignore[invalid-method-override]
         htmlsrc = self.do_encoding(htmlsrc)
         self.feed(htmlsrc)
 
@@ -222,36 +223,35 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         """Check if we're currently in an ignored section."""
         return self.ignore_depth > 0 or self.comment_ignore
 
-    def begin_translation_unit(self):
+    def begin_translation_unit(self) -> None:
         # at the start of a translation unit:
         # this interrupts any translation unit in progress, so process the queue
         # and prepare for the new.
         self.emit_translation_unit()
         self.tu_content = []
-        self.tu_location = "{}+{}:{}-{}".format(
-            self.filename,
-            ".".join(self.tag_path),
-            self.getpos()[0],
-            self.getpos()[1] + 1,
-        )
+        self.tu_location = f"{self.filename}+{'.'.join(self.tag_path)}:{self.getpos()[0]}-{self.getpos()[1] + 1}"
 
-    def end_translation_unit(self):
+    def end_translation_unit(self) -> None:
         # at the end of a translation unit:
         # process the queue and reset state.
         self.emit_translation_unit()
         self.tu_content = []
         self.tu_location = None
 
-    def append_markup(self, markup):
+    def append_markup(self, markup) -> None:
         # if within a translation unit: add to the queue to be processed later.
         # otherwise handle immediately.
-        if self.tu_location:
+        # However, if we're in an ignored section, output directly to filesrc
+        # to avoid including ignored content in parent translation units
+        if self.is_extraction_ignored():
+            self.filesrc += markup["html_content"]
+        elif self.tu_location:
             self.tu_content.append(markup)
         else:
             self.emit_attribute_translation_units(markup)
             self.filesrc += markup["html_content"]
 
-    def emit_translation_unit(self):
+    def emit_translation_unit(self) -> None:
         # If we're in an ignored section, just output the raw content
         if self.is_extraction_ignored():
             for markup in self.tu_content:
@@ -327,13 +327,10 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
                 normalized_content = html_content.strip()
             assert normalized_content  # shouldn't be here otherwise
 
-            unit = self.addsourceunit(normalized_content)
-            unit.addlocation(self.tu_location)
-
             # Determine context from data-translate-context (outermost wins)
             # tu_content starts at the unit's opening tag, so the first
             # translate_context encountered is the correct one.
-            context = next(
+            explicit_context = next(
                 (
                     m["translate_context"]
                     for m in self.tu_content
@@ -341,19 +338,34 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
                 ),
                 None,
             )
-            if context:
-                unit.setcontext(context)
+
+            # If an explicit context is present, reuse an existing unit with the
+            # same source+context instead of creating a duplicate unit.
+            unit = None
+            if explicit_context:
+                unit = self._units_by_src_ctx.get(
+                    (normalized_content, explicit_context)
+                )
+            if unit is None:
+                unit = self.addsourceunit(normalized_content)
+                if explicit_context:
+                    unit.setcontext(explicit_context)
+                    # Register for faster lookups
+                    self._units_by_src_ctx[normalized_content, explicit_context] = unit
+
+            unit.addlocation(self.tu_location)
+
             # If no explicit context, capture a context hint (from id/ancestor) for potential disambiguation
             context_hint = None
-            if not context:
+            if not explicit_context:
                 context_hint = next(
                     (m["context_hint"] for m in self.tu_content if "context_hint" in m),
                     None,
                 )
-            # Register the unit for potential disambiguation using context hints
-            self._register_unit_for_disambiguation(
-                unit, normalized_content, context_hint
-            )
+                # Register the unit for potential disambiguation using context hints
+                self._register_unit_for_disambiguation(
+                    unit, normalized_content, context_hint
+                )
 
             # Extract comment text from HTML comment elements within the translation unit
             comments = [
@@ -435,30 +447,33 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
             return {
                 "html_content": normalized_value,
                 "attrname": attrname,
-                "location": "{}+{}{}:{}-{}".format(
-                    self.filename,
-                    ".".join(self.tag_path),
-                    f"[{attrname}]",
-                    self.getpos()[0],
-                    self.getpos()[1] + 1,
-                ),
+                "location": f"{self.filename}+{'.'.join(self.tag_path)}[{attrname}]:{self.getpos()[0]}-{self.getpos()[1] + 1}",
             }
         return None
 
-    def emit_attribute_translation_units(self, markup):
+    def emit_attribute_translation_units(self, markup) -> None:
         if "attribute_tus" in markup:
             for tu in markup["attribute_tus"]:
-                unit = self.addsourceunit(tu["html_content"])
-                unit.addlocation(tu["location"])
                 # Attribute context: prefer explicit translate_context; otherwise use context_hint only if needed
                 attr_suffix = f"[{tu.get('attrname')}]" if tu.get("attrname") else ""
                 explicit = markup.get("translate_context")
-                if explicit:
-                    unit.setcontext(f"{explicit}{attr_suffix}")
-                    self._register_unit_for_disambiguation(
-                        unit, tu["html_content"], None
+                full_explicit = f"{explicit}{attr_suffix}" if explicit else None
+
+                # If explicit context exists, reuse an existing unit (source+context)
+                unit = None
+                if full_explicit:
+                    unit = self._units_by_src_ctx.get(
+                        (tu["html_content"], full_explicit)
                     )
-                else:
+                if unit is None:
+                    unit = self.addsourceunit(tu["html_content"])
+                    if full_explicit:
+                        unit.setcontext(full_explicit)
+                        # Register for faster lookups
+                        self._units_by_src_ctx[tu["html_content"], full_explicit] = unit
+                unit.addlocation(tu["location"])
+
+                if not full_explicit:
                     hint_base = markup.get("context_hint")
                     hint = f"{hint_base}{attr_suffix}" if hint_base else None
                     self._register_unit_for_disambiguation(
@@ -523,7 +538,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         return result
 
     @staticmethod
-    def create_start_tag(tag, attrs=None, startend=False):
+    def create_start_tag(tag, attrs=None, startend=False) -> str:
         attr_strings = []
         attrs = attrs or []
         for attrname, attrvalue in attrs:
@@ -531,9 +546,9 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
                 attr_strings.append(f" {attrname}")
             else:
                 attr_strings.append(f' {attrname}="{attrvalue}"')
-        return "<{}{}{}>".format(tag, "".join(attr_strings), " /" if startend else "")
+        return f"<{tag}{''.join(attr_strings)}{' /' if startend else ''}>"
 
-    def auto_close_empty_element(self):
+    def auto_close_empty_element(self) -> None:
         if self.tag_path and self.tag_path[-1] in self.EMPTY_HTML_ELEMENTS:
             self.tag_path.pop()
 
@@ -547,7 +562,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
 
     # From here on below, follows the methods of the HTMLParser
 
-    def handle_starttag(self, tag, attrs):
+    def handle_starttag(self, tag, attrs) -> None:
         self.auto_close_empty_element()
         self.tag_path.append(tag)
 
@@ -633,7 +648,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
 
         self.append_markup(markup)
 
-    def handle_endtag(self, tag):
+    def handle_endtag(self, tag) -> None:
         try:
             popped = self.tag_path.pop()
         except IndexError:
@@ -673,7 +688,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
                 if self._ancestor_id_label_stack:
                     self._ancestor_id_label_stack.pop()
 
-    def handle_startendtag(self, tag, attrs):
+    def handle_startendtag(self, tag, attrs) -> None:
         self.auto_close_empty_element()
         self.tag_path.append(tag)
 
@@ -776,7 +791,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
                 if self._ancestor_id_label_stack:
                     self._ancestor_id_label_stack.pop()
 
-    def _register_unit_for_disambiguation(self, unit, source, context_hint):
+    def _register_unit_for_disambiguation(self, unit, source, context_hint) -> None:
         bucket = self._units_by_source.setdefault(source, [])
         bucket.append((unit, context_hint))
         if len(bucket) == 2:
@@ -786,18 +801,18 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         elif len(bucket) > 2 and context_hint and not unit.getcontext():
             unit.setcontext(context_hint)
 
-    def handle_data(self, data):
+    def handle_data(self, data) -> None:
         self.auto_close_empty_element()
         self.append_markup({"type": "data", "html_content": data})
 
-    def handle_charref(self, name):
+    def handle_charref(self, name) -> None:
         """Handle entries in the form &#NNNN; e.g. &#8417;."""
         if name.lower().startswith("x"):
             self.handle_data(chr(int(name[1:], 16)))
         else:
             self.handle_data(chr(int(name)))
 
-    def handle_entityref(self, name):
+    def handle_entityref(self, name) -> None:
         """Handle named entities of the form &aaaa; e.g. &rsquo;."""
         converted = html5.get(f"{name};")
         if name in {"gt", "lt", "amp"} or not converted:
@@ -805,7 +820,7 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
         else:
             self.handle_data(converted)
 
-    def handle_comment(self, data):
+    def handle_comment(self, data) -> None:
         self.auto_close_empty_element()
 
         # Check for translate:off and translate:on directives
@@ -819,15 +834,15 @@ class htmlfile(html.parser.HTMLParser, base.TranslationStore):
             {"type": "comment", "html_content": f"<!--{data}-->", "note": data}
         )
 
-    def handle_decl(self, decl):
+    def handle_decl(self, decl) -> None:
         self.auto_close_empty_element()
         self.append_markup({"type": "decl", "html_content": f"<!{decl}>"})
 
-    def handle_pi(self, data):
+    def handle_pi(self, data) -> None:
         self.auto_close_empty_element()
         self.append_markup({"type": "pi", "html_content": f"<?{data}?>"})
 
-    def unknown_decl(self, data):
+    def unknown_decl(self, data) -> None:
         self.auto_close_empty_element()
         self.append_markup({"type": "cdecl", "html_content": f"<![{data}]>"})
 
