@@ -21,6 +21,7 @@
 from __future__ import annotations
 
 import contextlib
+from typing import TypeVar
 
 from lxml import etree
 
@@ -226,7 +227,7 @@ class LISAunit(base.TranslationUnit):
                 if getXMLlang(set) == lang:
                     return set
         else:  # have to use index
-            if index >= len(languageNodes):
+            if index >= len(languageNodes):  # ty:ignore[unsupported-operator]
                 return None
             return languageNodes[index]
         return None
@@ -270,7 +271,10 @@ class LISAunit(base.TranslationUnit):
         return term
 
 
-class LISAfile(base.TranslationStore):
+U = TypeVar("U", bound=LISAunit)
+
+
+class LISAfile(base.TranslationStore[U]):
     """A class representing a file store for one of the LISA file formats."""
 
     UnitClass = LISAunit
@@ -338,7 +342,7 @@ class LISAfile(base.TranslationStore):
         unit.namespace = self.namespace
         super().addunit(unit)
         if new:
-            self.body.append(unit.xmlelement)
+            self.body.append(unit.xmlelement)  # ty:ignore[unresolved-attribute]
 
     def removeunit(self, unit) -> None:
         super().removeunit(unit)
@@ -375,9 +379,9 @@ class LISAfile(base.TranslationStore):
             doctype=self.XMLdoctype,
         )
 
-        out.write(self.serialize_hook(treestring))
+        out.write(self.serialize_hook(treestring))  # ty:ignore[invalid-argument-type]
 
-    def parse(self, xml) -> None:
+    def parse(self, xml) -> None:  # ty:ignore[invalid-method-override]
         """Populates this object from the given xml string."""
         if not hasattr(self, "filename"):
             self.filename = getattr(xml, "name", "")
