@@ -23,19 +23,12 @@ This file contains abstract (semantic) interfaces for placeable
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-
 from translate.storage.placeables.strelem import StringElem
 
-if TYPE_CHECKING:
-    from collections.abc import Callable
-
 
 class BasePlaceable(StringElem):
     """Base class for all placeables."""
 
-    parse: Callable[[str], list[StringElem]]
-
 
 class InvisiblePlaceable(BasePlaceable):
     pass
