@@ -113,7 +113,7 @@ class tsunit(lisa.LISAunit):
     _context = None
     _locations = None
 
-    def createlanguageNode(self, lang, text, purpose):
+    def createlanguageNode(self, lang, text, purpose):  # ty:ignore[invalid-method-override]
         """Returns an xml Element setup with given parameters."""
         assert purpose
         if purpose == "target":
@@ -292,7 +292,7 @@ class tsunit(lisa.LISAunit):
             return None
         return context.text
 
-    def setcontext(self, value) -> None:
+    def setcontext(self, value) -> None:  # ty:ignore[invalid-method-override]
         if value == self.getcontextname():
             return
         parent = self.xmlelement.getparent()
@@ -365,7 +365,7 @@ class tsunit(lisa.LISAunit):
 
     def get_previous_unit(self):
         found = None
-        for pos, unit in enumerate(self._store.units):
+        for pos, unit in enumerate(self._store.units):  # ty:ignore[unresolved-attribute]
             # Use is here to compare objects as __eq__ implementation in
             # LISAUnit might give unexpected results
             if unit is self:
@@ -373,7 +373,7 @@ class tsunit(lisa.LISAunit):
                 break
         if not found or found == 0:
             return None
-        return self._store.units[found - 1]
+        return self._store.units[found - 1]  # ty:ignore[unresolved-attribute]
 
     def getlocations(self):
         if self._locations is None:
@@ -381,7 +381,7 @@ class tsunit(lisa.LISAunit):
 
         return [
             f"{location}{':' if location else ''}{line}"
-            for location, line in self._locations
+            for location, line in self._locations  # ty:ignore[not-iterable]
         ]
 
     def merge(
@@ -420,7 +420,7 @@ class tsunit(lisa.LISAunit):
         self._settype(self.statemap_r[value])
 
 
-class tsfile(lisa.LISAfile):
+class tsfile(lisa.LISAfile[tsunit]):
     """Class representing a TS file store."""
 
     UnitClass = tsunit
@@ -507,7 +507,7 @@ class tsfile(lisa.LISAfile):
     def _getcontextnames(self):
         """Returns all contextnames in this TS file."""
         contextnodes = self.document.findall(self.namespaced("context"))
-        return [self.getcontextname(contextnode) for contextnode in contextnodes]
+        return [self.getcontextname(contextnode) for contextnode in contextnodes]  # ty:ignore[unresolved-attribute]
 
     def _getcontextnode(self, contextname):
         """Returns the context node with the given name."""
@@ -553,7 +553,7 @@ class tsfile(lisa.LISAfile):
         return self.body is not None
 
     def nplural(self):
-        code = self.header.get("language").lower().replace("-", "_").split("_")[0]
+        code = self.header.get("language").lower().replace("-", "_").split("_")[0]  # ty:ignore[unresolved-attribute]
         if code in data.qt_plural_tags:
             return len(data.qt_plural_tags[code])
         lang = data.get_language(self.header.get("language"))
