@@ -23,6 +23,7 @@ The official recommendation is to use the extension .xlf for XLIFF files.
 """
 
 import contextlib
+from typing import TypeVar
 
 from lxml import etree
 
@@ -164,7 +165,7 @@ class Xliff1Unit(XliffUnit):
                 )
                 # TODO: support multiple targets better
                 # TODO: support notes in alt-trans
-                newunit.xmlelement = node
+                newunit.xmlelement = node  # ty:ignore[unresolved-attribute]
 
                 translist.append(newunit)
         return translist
@@ -354,7 +355,7 @@ class Xliff1Unit(XliffUnit):
         if state_id < self.S_UNREVIEWED:
             self.set_state_n(self.S_UNREVIEWED)
 
-    def setid(self, id) -> None:
+    def setid(self, id) -> None:  # ty:ignore[invalid-method-override]
         # sanitize id in case ID_SEPARATOR is present
         self.xmlelement.set("id", id.replace(ID_SEPARATOR, ID_SEPARATOR_SAFE))
 
@@ -473,7 +474,10 @@ class Xliff1Unit(XliffUnit):
         )
 
 
-class Xliff1File(XliffFile):
+U = TypeVar("U", bound=Xliff1Unit)
+
+
+class Xliff1File(XliffFile[U]):
     """Class representing a XLIFF file store."""
 
     UnitClass = Xliff1Unit
@@ -523,7 +527,7 @@ class Xliff1File(XliffFile):
 
     def createfilenode(
         self, filename, sourcelanguage=None, targetlanguage=None, datatype="plaintext"
-    ):
+    ) -> etree.Element:
         """
         Creates a filenode with the given filename. All parameters are
         needed for XLIFF compliance.
@@ -552,7 +556,7 @@ class Xliff1File(XliffFile):
         return filenode
 
     @staticmethod
-    def getfilename(filenode):
+    def getfilename(filenode) -> str | None:
         """Returns the name of the given file."""
         return filenode.get("original")
 
@@ -561,7 +565,7 @@ class Xliff1File(XliffFile):
         """Set the name of the given file."""
         return filenode.set("original", filename)
 
-    def setsourcelanguage(self, language) -> None:
+    def setsourcelanguage(self, language) -> None:  # ty:ignore[invalid-method-override]
         if not language:
             return
         for filenode in self.document.getroot().iterchildren(self.namespaced("file")):
@@ -573,7 +577,7 @@ class Xliff1File(XliffFile):
 
     sourcelanguage = property(getsourcelanguage, setsourcelanguage)
 
-    def settargetlanguage(self, language) -> None:
+    def settargetlanguage(self, language) -> None:  # ty:ignore[invalid-method-override]
         if not language:
             return
         for filenode in self.document.getroot().iterchildren(self.namespaced("file")):
@@ -686,7 +690,7 @@ class Xliff1File(XliffFile):
                 target_group.append(unit.xmlelement)
             else:
                 # Add directly to body
-                self.body.append(unit.xmlelement)
+                self.body.append(unit.xmlelement)  # ty:ignore[unresolved-attribute]
 
     def addsourceunit(self, source, filename="NoName", createifmissing=False):
         """
@@ -727,7 +731,7 @@ class Xliff1File(XliffFile):
         """Adds a group tag into the specified file."""
         if not self.switchfile(filename, createifmissing):
             return None
-        group = etree.SubElement(self.body, self.namespaced("group"))
+        group = etree.SubElement(self.body, self.namespaced("group"))  # ty:ignore[no-matching-overload]
         if restype:
             group.set("restype", restype)
         return group
@@ -740,13 +744,13 @@ class Xliff1File(XliffFile):
         :returns: The matching or newly created group element
         """
         # Try to find a matching group in the current body
-        for group in self.body.iterchildren(self.namespaced("group")):
+        for group in self.body.iterchildren(self.namespaced("group")):  # ty:ignore[unresolved-attribute]
             # Check if all attributes match
             if group.attrib == source_group.attrib:
                 return group
 
         # No matching group found, create a new one
-        group = etree.SubElement(self.body, self.namespaced("group"))
+        group = etree.SubElement(self.body, self.namespaced("group"))  # ty:ignore[no-matching-overload]
         for attr, value in source_group.attrib.items():
             group.set(attr, value)
         return group
